#! /usr/bin/env python3

import glob
import sys
import math

################################################################################
if __name__ == "__main__" :

  scoreTypes = {
    "F1" : ["F1.score","%",-1,"%.2f"],
    "R2" : ["R²","",-3,"%.4f"],
    "L1" : ["L1","",-1,"%.2f"],
    "L2" : ["L2","",-2,"%.2f"],
  }

  scoreType = scoreTypes["F1"] if len(sys.argv) < 3 else scoreTypes[sys.argv[2].upper()]

  metrics = ["LAS","UAS","Tokens","Words","Sentences","UPOS","UFeats","Lemmas"]

  output = []
  outputByModelScore = dict()

  filenamesErr = glob.iglob((sys.argv[1]+"/"if len(sys.argv) > 1 else "") + '*stderr')
  filenamesOut = glob.iglob((sys.argv[1]+"/"if len(sys.argv) > 1 else "") + '*stdout')

  for pathToFile in filenamesErr :
    for line in open(pathToFile, "r") :
      if "Error" in line or "ERROR" in line or "error" in line :
        print(pathToFile,":", file=sys.stderr)
        print("\t"+line,end="", file=sys.stderr)

  for pathToFile in  filenamesOut :
    splited = pathToFile.split('/')[-1].split('.')
    model = ".".join(splited[:-3])
    corpus = splited[-3]
    index = splited[-2]

    if corpus not in outputByModelScore :
      outputByModelScore[corpus] = dict()

    for line in open(pathToFile, "r") :
      for metric in metrics :
        if metric in line and metric[0] == line[0]:
          splited = line.strip().replace("|","").split()

          if model not in outputByModelScore[corpus] :
            outputByModelScore[corpus][model] = dict()
          if splited[0] not in outputByModelScore[corpus][model] :
            outputByModelScore[corpus][model][splited[0]] = []

          outputByModelScore[corpus][model][splited[0]].append([corpus, splited[0], splited[scoreType[2]], model])

  for corpus in outputByModelScore :
    for model in outputByModelScore[corpus] :
      for metric in outputByModelScore[corpus][model] :
        score = 0.0
        for exp in outputByModelScore[corpus][model][metric] :
          score += float(exp[2])
        score /= len(outputByModelScore[corpus][model][metric])
        standardDeviation = 0.0
        if len(outputByModelScore[corpus][model][metric]) > 1 :
          for exp in outputByModelScore[corpus][model][metric] :
            standardDeviation += (float(exp[2])-score)**2
          standardDeviation /= len(outputByModelScore[corpus][model][metric])
          standardDeviation = math.sqrt(standardDeviation)
        baseScore = score
        if float(scoreType[3]%standardDeviation) > 0 :
          score = "%s[±%s]%%s"%(scoreType[3],scoreType[3])%(score,standardDeviation,scoreType[1])
        else :
          score = "%s%%s"%scoreType[3]%(score, scoreType[1])
        if '-' in score :
          score = score.replace('-','')
        output.append(outputByModelScore[corpus][model][metric][0])
        output[-1][2] = score
        output[-1] = [output[-1][0]] + [baseScore] + output[-1][1:]

  if len(output) == 0 :
    print("ERROR : Output length is 0", file=sys.stderr)
    print("  did you run evaluate.sh ?", file=sys.stderr)
    print("USAGE : %s [directory of .stdout files]"%sys.argv[0], file=sys.stderr)
    exit(1)

  output.sort()
  output = [[val[0]] + val[2:] for val in output]

  maxColLens = [0 for _ in range(len(output[0]))]

  output = [["Corpus","Metric",scoreType[0],"Model"]] + output

  for line in output :
    for i in range(len(line)) :
      maxColLens[i] = max(maxColLens[i], len(str(line[i])))

  dashLine = '-' * 80
  for i in range(len(output)) :
    if i > 0 and output[i][0] != output[i-1][0] :
      print(dashLine)
    elif i > 0 and output[i][1] != output[i-1][1] :
      print("")
    for j in range(len(output[i])) :
      padding = (' '*(maxColLens[j]-len(str(output[i][j]))))+" "*3
      print(output[i][j], end=padding)
    print("")
################################################################################

